"use strict";
const tslib_1 = require("tslib");
const exports_1 = require("parcel-plugin-typescript/exports");
const client_1 = require("../../backend/worker/client");
const collect_dependencies_1 = require("../utils/collect-dependencies");
module.exports = class AngularAOTTSAsset extends exports_1.JSAsset {
    constructor(name, pkg, options) {
        super(name, pkg, options);
        this.resources = null;
        this.tsDependencies = null;
        this.config = exports_1.loadConfiguration(name);
    }
    mightHaveDependencies() {
        return true;
    }
    pretransform() {
        const _super = name => super[name];
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            if (!this.tsDependencies) {
                return _super("pretransform").call(this);
            }
        });
    }
    transform() {
        const _super = name => super[name];
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            if (!this.tsDependencies || this.options.minify) {
                return _super("transform").call(this);
            }
        });
    }
    generate() {
        if (!this.tsDependencies) {
            return super.generate();
        }
        return {
            js: this.contents
        };
    }
    collectDependencies() {
        collect_dependencies_1.collectDependencies(this, this.resources);
        if (!this.tsDependencies) {
            return super.collectDependencies();
        }
        this.tsDependencies.forEach(dep => this.addDependency(dep.source, { loc: dep.position }));
    }
    parse() {
        const _super = name => super[name];
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const { path: tsConfig } = yield this.config;
            const result = yield client_1.IPCClient.compile({ tsConfig, file: this.name });
            this.resources = result.resources;
            this.contents = result.sources.js;
            this.tsDependencies = result.dependencies;
            if (!this.tsDependencies) {
                // Parse result as ast format through babylon
                return _super("parse").call(this, this.contents);
            }
            return { blank: true };
        });
    }
};
//# sourceMappingURL=aot.js.map