import {Configuration, ImportDependency, JSAsset, loadConfiguration} from 'parcel-plugin-typescript/exports'

import {IPCClient} from '../../backend/worker/client'
import {Resources} from '../../interfaces'

import {collectDependencies} from '../utils/collect-dependencies'

export = class AngularAOTTSAsset extends JSAsset {
	private readonly config: Promise<Configuration>
	private resources: Resources|null = null
	private tsDependencies: ImportDependency[]|null = null

	constructor(name: string, pkg: string, options: any) {
		super(name, pkg, options)

		this.config = loadConfiguration(name)
	}

	public mightHaveDependencies() {
		return true
	}

	public async pretransform() {
		if(!this.tsDependencies) {
			return super.pretransform()
		}
	}

	public async transform() {
		if(!this.tsDependencies || this.options.minify) {
			return super.transform()
		}
	}

	public generate() {
		if(!this.tsDependencies) {
			return super.generate()
		}

		return {
			js: this.contents!
		}
	}

	public collectDependencies() {
		collectDependencies(this, this.resources)

		if(!this.tsDependencies) {
			return super.collectDependencies()
		}

		this.tsDependencies.forEach(dep =>
			this.addDependency(dep.source, {loc: dep.position})
		)
	}

	public async parse() {
		const {path: tsConfig} = await this.config
		const result = await IPCClient.compile({tsConfig, file: this.name})

		this.resources = result.resources
		this.contents = result.sources.js
		this.tsDependencies = result.dependencies

		if(!this.tsDependencies) {
			// Parse result as ast format through babylon
			return super.parse(this.contents)
		}

		return {blank: true}
	}
}
